@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 #
-# Copyright (c), 2016-2021, SISSA (International School for Advanced Studies).
+# Copyright (c), 2016-2022, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -9,6 +8,7 @@
 # @author Davide Brunato <brunato@sissa.it>
 #
 from . import limits
+from . import translation
 from .exceptions import XMLSchemaException, XMLResourceError, XMLSchemaNamespaceError
 from .etree import ElementData, etree_tostring
 from .resources import normalize_url, normalize_locations, fetch_resource, \
@@ -30,27 +30,26 @@ from .validators import (
     XsdComponent, XsdType, XsdElement, XsdAttribute
 )
 
-__version__ = '1.9.1'
+__version__ = '1.11.3'
 __author__ = "Davide Brunato"
 __contact__ = "brunato@sissa.it"
-__copyright__ = "Copyright 2016-2021, SISSA"
+__copyright__ = "Copyright 2016-2022, SISSA"
 __license__ = "MIT"
 __status__ = "Production/Stable"
 
-
 __all__ = [
-    'limits', 'XMLSchemaException', 'XMLResourceError', 'XMLSchemaNamespaceError',
-    'etree_tostring', 'normalize_url', 'normalize_locations', 'fetch_resource',
-    'fetch_namespaces', 'fetch_schema_locations', 'fetch_schema', 'XMLResource',
-    'ElementPathMixin', 'ElementData', 'XMLSchemaConverter', 'UnorderedConverter',
-    'ParkerConverter', 'BadgerFishConverter', 'AbderaConverter', 'JsonMLConverter',
-    'ColumnarConverter', 'DataElement', 'DataElementConverter', 'DataBindingConverter',
-    'validate', 'is_valid', 'iter_errors', 'iter_decode', 'to_dict', 'to_json',
-    'from_json', 'XmlDocument', 'XMLSchemaValidatorError', 'XMLSchemaParseError',
-    'XMLSchemaNotBuiltError', 'XMLSchemaModelError', 'XMLSchemaModelDepthError',
-    'XMLSchemaValidationError', 'XMLSchemaDecodeError', 'XMLSchemaEncodeError',
-    'XMLSchemaChildrenValidationError', 'XMLSchemaIncludeWarning',
-    'XMLSchemaImportWarning', 'XMLSchemaTypeTableWarning',
-    'XsdGlobals', 'XMLSchemaBase', 'XMLSchema', 'XMLSchema10', 'XMLSchema11',
-    'XsdComponent', 'XsdType', 'XsdElement', 'XsdAttribute',
+    'limits', 'translation', 'XMLSchemaException', 'XMLResourceError',
+    'XMLSchemaNamespaceError', 'etree_tostring', 'normalize_url', 'normalize_locations',
+    'fetch_resource', 'fetch_namespaces', 'fetch_schema_locations', 'fetch_schema',
+    'XMLResource', 'ElementPathMixin', 'ElementData', 'XMLSchemaConverter',
+    'UnorderedConverter', 'ParkerConverter', 'BadgerFishConverter', 'AbderaConverter',
+    'JsonMLConverter', 'ColumnarConverter', 'DataElement', 'DataElementConverter',
+    'DataBindingConverter', 'validate', 'is_valid', 'iter_errors', 'iter_decode',
+    'to_dict', 'to_json', 'from_json', 'XmlDocument', 'XMLSchemaValidatorError',
+    'XMLSchemaParseError', 'XMLSchemaNotBuiltError', 'XMLSchemaModelError',
+    'XMLSchemaModelDepthError', 'XMLSchemaValidationError', 'XMLSchemaDecodeError',
+    'XMLSchemaEncodeError', 'XMLSchemaChildrenValidationError', 'XMLSchemaIncludeWarning',
+    'XMLSchemaImportWarning', 'XMLSchemaTypeTableWarning', 'XsdGlobals', 'XMLSchemaBase',
+    'XMLSchema', 'XMLSchema10', 'XMLSchema11', 'XsdComponent', 'XsdType', 'XsdElement',
+    'XsdAttribute',
 ]
