@@ -31,8 +31,8 @@ from .aliases import ElementType, ElementTreeType, NamespacesType, XMLSourceType
 from .helpers import get_namespace, is_etree_element, is_etree_document, \
     etree_iter_location_hints
 
-DEFUSE_MODES = frozenset(('never', 'remote', 'always'))
-SECURITY_MODES = frozenset(('all', 'remote', 'local', 'sandbox'))
+DEFUSE_MODES = frozenset(('never', 'remote', 'nonlocal', 'always'))
+SECURITY_MODES = frozenset(('all', 'remote', 'local', 'sandbox', 'none'))
 
 ###
 # Restricted XPath parser for XML resources
@@ -133,7 +133,7 @@ class _PurePath(PurePath):
 
             obj = _PureWindowsPath(unquote(path))
             if len(obj.drive) != 2 or obj.drive[1] != ':':
-                raise XMLSchemaValueError("Invalid URI {!r}".format(uri))
+                raise XMLSchemaValueError("Invalid URI %r" % uri)
             return obj
 
         if '\\' in path:
@@ -334,7 +334,7 @@ def fetch_resource(location: str, base_url: Optional[str] = None, timeout: int =
     :return: a normalized URL.
     """
     if not location:
-        raise XMLSchemaValueError("'location' argument must contain a not empty string")
+        raise XMLSchemaValueError("'location' argument should contain a not empty string")
 
     url = normalize_url(location, base_url)
     try:
@@ -344,13 +344,15 @@ def fetch_resource(location: str, base_url: Optional[str] = None, timeout: int =
         # fallback joining the path without a base URL
         alt_url = normalize_url(location)
         if url == alt_url:
-            raise XMLResourceError("cannot access to resource %r: %s" % (url, err.reason))
+            msg = "cannot access to resource %(url)r: %(reason)s"
+            raise XMLResourceError(msg % {'url': url, 'reason': err.reason})
 
         try:
             with urlopen(alt_url, timeout=timeout):
                 return alt_url
         except URLError:
-            raise XMLResourceError("cannot access to resource %r: %s" % (url, err.reason))
+            msg = "cannot access to resource %(url)r: %(reason)s"
+            raise XMLResourceError(msg % {'url': url, 'reason': err.reason})
 
 
 def fetch_schema_locations(source: Union['XMLResource', XMLSourceType],
@@ -384,8 +386,8 @@ def fetch_schema_locations(source: Union['XMLResource', XMLSourceType],
     namespace = resource.namespace
     locations = resource.get_locations(locations, root_only=False)
     if not locations:
-        msg = "{!r} does not contain any schema location hint"
-        raise XMLSchemaValueError(msg.format(source))
+        msg = "%r does not contain any schema location hint"
+        raise XMLSchemaValueError(msg % source)
 
     for ns, url in sorted(locations, key=lambda x: x[0] != namespace):
         try:
@@ -393,7 +395,7 @@ def fetch_schema_locations(source: Union['XMLResource', XMLSourceType],
         except XMLResourceError:
             pass
 
-    raise XMLSchemaValueError("not found a schema for XML data resource {!r}.".format(source))
+    raise XMLSchemaValueError("not found a schema for XML data resource %r" % source)
 
 
 def fetch_schema(source: Union['XMLResource', XMLSourceType],
@@ -434,12 +436,15 @@ class XMLResource:
     when the URL of the resource can't be obtained from the source argument. For security \
     access to a local file resource is always denied if the *base_url* is a remote URL.
     :param allow: defines the security mode for accessing resource locations. Can be \
-    'all', 'remote', 'local' or 'sandbox'. Default is 'all' that means all types of \
+    'all', 'remote', 'local', 'sandbox' or 'none'. Default is 'all' that means all types of \
     URLs are allowed. With 'remote' only remote resource URLs are allowed. With 'local' \
     only file paths and URLs are allowed. With 'sandbox' only file paths and URLs that \
-    are under the directory path identified by the *base_url* argument are allowed.
+    are under the directory path identified by the *base_url* argument are allowed. \
+    If you provide 'none' no located resource is allowed.
     :param defuse: defines when to defuse XML data using a `SafeXMLParser`. Can be \
-    'always', 'remote' or 'never'. For default defuses only remote XML data.
+    'always', 'remote', 'nonlocal' or 'never'. For default defuses only remote XML data. \
+    With 'always' all the XML data that is not already parsed is defused. With 'nonlocal' \
+    it defuses unparsed data except local files. With 'never' no XML data source is defused.
     :param timeout: the timeout in seconds for the connection attempt in case of remote data.
     :param lazy: if a value `False` or 0 is provided the XML data is fully loaded into and \
     processed from memory. For default only the root element of the source is loaded, \
@@ -475,31 +480,31 @@ class XMLResource:
                 raise XMLSchemaValueError("'base_url' argument is not an URL")
             self._base_url = base_url.decode()
         elif base_url is not None:
-            msg = "invalid type {!r} for argument 'base_url'"
-            raise XMLSchemaTypeError(msg.format(type(base_url)))
+            msg = "invalid type %r for argument 'base_url'"
+            raise XMLSchemaTypeError(msg % type(base_url))
 
         if not isinstance(allow, str):
-            msg = "invalid type {!r} for argument 'allow'"
-            raise XMLSchemaTypeError(msg.format(type(allow)))
+            msg = "invalid type %r for argument 'allow'"
+            raise XMLSchemaTypeError(msg % type(allow))
         elif allow not in SECURITY_MODES:
-            msg = "'allow' argument: {!r} is not a security mode"
-            raise XMLSchemaValueError(msg.format(allow))
+            msg = "'allow' argument: %r is not a security mode"
+            raise XMLSchemaValueError(msg % allow)
         elif allow == 'sandbox' and self._base_url is None:
             msg = "block access to files out of sandbox requires 'base_url' to be set"
             raise XMLResourceError(msg)
         self._allow = allow
 
         if not isinstance(defuse, str):
-            msg = "invalid type {!r} for argument 'defuse'"
-            raise XMLSchemaTypeError(msg.format(type(defuse)))
+            msg = "invalid type %r for argument 'defuse'"
+            raise XMLSchemaTypeError(msg % type(defuse))
         elif defuse not in DEFUSE_MODES:
-            msg = "'defuse' argument: {!r} is not a defuse mode"
-            raise XMLSchemaValueError(msg.format(defuse))
+            msg = "'defuse' argument: %r is not a defuse mode"
+            raise XMLSchemaValueError(msg % defuse)
         self._defuse = defuse
 
         if not isinstance(timeout, int):
-            msg = "invalid type {!r} for argument 'timeout'"
-            raise XMLSchemaTypeError(msg.format(type(timeout)))
+            msg = "invalid type %r for argument 'timeout'"
+            raise XMLSchemaTypeError(msg % type(timeout))
         elif timeout <= 0:
             msg = "the argument 'timeout' must be a positive integer"
             raise XMLSchemaValueError(msg)
@@ -573,6 +578,8 @@ class XMLResource:
     def _access_control(self, url: str) -> None:
         if self._allow == 'all':
             return
+        elif self._allow == 'none':
+            raise XMLResourceError("block access to resource {}".format(url))
         elif self._allow == 'remote':
             if is_local_url(url):
                 raise XMLResourceError("block access to local resource {}".format(url))
@@ -622,6 +629,7 @@ class XMLResource:
                 _nsmap = []
 
         if self._defuse == 'remote' and is_remote_url(self.base_url) \
+                or self._defuse == 'nonlocal' and not is_local_url(self.base_url) \
                 or self._defuse == 'always':
             safe_parser = SafeXMLParser(target=PyElementTree.TreeBuilder())
             tree_iterator = PyElementTree.iterparse(resource, events, safe_parser)
@@ -631,7 +639,7 @@ class XMLResource:
         root_started = False
         nsmap_update = False
 
-        _root: ElementType = getattr(self, '_root', None)
+        _root = cast(Optional[ElementType], getattr(self, '_root', None))
 
         try:
             for event, node in tree_iterator:
@@ -663,6 +671,7 @@ class XMLResource:
 
     def _parse(self, resource: IO[AnyStr]) -> None:
         if self._defuse == 'remote' and is_remote_url(self.base_url) \
+                or self._defuse == 'nonlocal' and not is_local_url(self.base_url) \
                 or self._defuse == 'always':
 
             if not hasattr(resource, 'seekable') or not resource.seekable():
@@ -726,11 +735,11 @@ class XMLResource:
         if isinstance(lazy, bool):
             pass
         elif not isinstance(lazy, int):
-            msg = "invalid type {!r} for the attribute 'lazy'"
-            raise XMLSchemaTypeError(msg.format(type(lazy)))
+            msg = "invalid type %r for the attribute 'lazy'"
+            raise XMLSchemaTypeError(msg % type(lazy))
         elif lazy < 0:
-            msg = "invalid value {!r} for the attribute 'lazy'"
-            raise XMLSchemaValueError(msg.format(lazy))
+            msg = "invalid value %r for the attribute 'lazy'"
+            raise XMLSchemaValueError(msg % lazy)
 
         url: Optional[str]
         if isinstance(source, str):
@@ -787,10 +796,10 @@ class XMLResource:
             # source is a readable resource (remote or local file)
             url = getattr(source, 'url', None)
             if url is not None:
+                self._access_control(url)
+
                 # Save remote urls for open new resources (non seekable)
-                if is_remote_url(url):
-                    self._access_control(url)
-                else:
+                if not is_remote_url(url):
                     url = None
 
             self._parse_resource(cast(IO[str], source), url, lazy)
@@ -890,9 +899,8 @@ class XMLResource:
         namespaces = self.get_namespaces(root_only=False)
         _string = etree_tostring(elem, namespaces, indent, max_lines,
                                  spaces_for_tab, xml_declaration)
-        if isinstance(_string, bytes):
-            return _string.decode('utf-8')
-        return _string
+
+        return _string.decode('utf-8') if isinstance(_string, bytes) else _string
 
     def subresource(self, elem: ElementType) -> 'XMLResource':
         """Create an XMLResource instance from a subelement of a non-lazy XML tree."""
@@ -938,9 +946,8 @@ class XMLResource:
         try:
             return cast(IO[AnyStr], urlopen(self._url, timeout=self._timeout))
         except URLError as err:
-            raise XMLResourceError(
-                "cannot access to resource %r: %s" % (self._url, err.reason)
-            )
+            msg = "cannot access to resource %(url)r: %(reason)s"
+            raise XMLResourceError(msg % {'url': self._url, 'reason': err.reason})
 
     def seek(self, position: int) -> Optional[int]:
         """
@@ -1024,8 +1031,7 @@ class XMLResource:
         """Returns `True` if the XML text of the data source is loaded."""
         return self._text is not None
 
-    def iter(self, tag: Optional[str] = None,
-             nsmap: Union[None, List[Tuple[str, str]], MutableMapping[str, str]] = None) \
+    def iter(self, tag: Optional[str] = None, nsmap: Optional[NsmapType] = None) \
             -> Iterator[ElementType]:
         """
         XML resource tree iterator. The iteration of a lazy resource
@@ -1285,10 +1291,10 @@ class XMLResource:
 
         try:
             if root_only or root_only is None and self._lazy:
-                for _ in self.iter(nsmap=namespaces):
+                for _elem in self.iter(nsmap=namespaces):
                     break
             else:
-                for _ in self.iter(nsmap=namespaces):
+                for _elem in self.iter(nsmap=namespaces):
                     pass
         except (ElementTree.ParseError, PyElementTree.ParseError, UnicodeEncodeError):
             pass
