@@ -12,30 +12,37 @@ This module contains classes for other XML Schema identity constraints.
 """
 import re
 import math
-from collections import Counter
-from elementpath import XPath2Parser, ElementPathError, XPathContext, translate_pattern
+from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Optional, Pattern, \
+    Tuple, Union, Counter
+from elementpath import XPath2Parser, ElementPathError, XPathToken, XPathContext, \
+    translate_pattern, datatypes
 
 from ..exceptions import XMLSchemaTypeError, XMLSchemaValueError
-from ..qnames import XSD_ANNOTATION, XSD_QNAME, XSD_UNIQUE, XSD_KEY, XSD_KEYREF, \
-    XSD_SELECTOR, XSD_FIELD, get_qname, get_extended_qname, is_not_xsd_annotation
+from ..names import XSD_QNAME, XSD_UNIQUE, XSD_KEY, XSD_KEYREF, XSD_SELECTOR, XSD_FIELD
+from ..translation import gettext as _
+from ..helpers import get_qname, get_extended_qname
+from ..aliases import ElementType, SchemaType, NamespacesType, AtomicValueType
 from ..xpath import iter_schema_nodes
-from .exceptions import XMLSchemaValidationError
 from .xsdbase import XsdComponent
 from .attributes import XsdAttribute
 
-QNAME_PATTERN = re.compile(
-    r'(?:(?P<prefix>[^\d\W][\w\-.\u00B7\u0300-\u036F\u0387\u06DD\u06DE\u203F\u2040]*):)?'
-    r'(?P<local>[^\d\W][\w\-.\u00B7\u0300-\u036F\u0387\u06DD\u06DE\u203F\u2040]*)',
-)
+if TYPE_CHECKING:
+    from .elements import XsdElement
 
-XSD_IDENTITY_XPATH_SYMBOLS = {
+IdentityFieldItemType = Union[AtomicValueType, XsdAttribute, Tuple[Any, ...], None]
+IdentityCounterType = Tuple[IdentityFieldItemType, ...]
+IdentityMapType = Dict[Union['XsdKey', 'XsdKeyref', str, None],
+                       Union['IdentityCounter', 'KeyrefCounter']]
+
+XSD_IDENTITY_XPATH_SYMBOLS = frozenset((
     'processing-instruction', 'following-sibling', 'preceding-sibling',
     'ancestor-or-self', 'attribute', 'following', 'namespace', 'preceding',
     'ancestor', 'position', 'comment', 'parent', 'child', 'false', 'text', 'node',
-    'true', 'last', 'not', 'and', 'mod', 'div', 'or', '..', '//', '!=', '<=', '>=', '(', ')',
-    '[', ']', '.', '@', ',', '/', '|', '*', '-', '=', '+', '<', '>', ':', '(end)', '(name)',
-    '(string)', '(float)', '(decimal)', '(integer)', '::', '{', '}',
-}
+    'true', 'last', 'not', 'and', 'mod', 'div', 'or', '..', '//', '!=', '<=', '>=',
+    '(', ')', '[', ']', '.', '@', ',', '/', '|', '*', '-', '=', '+', '<', '>', ':',
+    '(end)', '(unknown)', '(invalid)', '(name)', '(string)', '(float)', '(decimal)',
+    '(integer)', '::', '{', '}',
+))
 
 
 # XSD identities use a restricted parser and a context for iterate element
@@ -47,42 +54,49 @@ class IdentityXPathContext(XPathContext):
 
 class IdentityXPathParser(XPath2Parser):
     symbol_table = {
-        k: v for k, v in XPath2Parser.symbol_table.items() if k in XSD_IDENTITY_XPATH_SYMBOLS
+        k: v for k, v in XPath2Parser.symbol_table.items()  # type: ignore[misc]
+        if k in XSD_IDENTITY_XPATH_SYMBOLS
     }
     SYMBOLS = XSD_IDENTITY_XPATH_SYMBOLS
 
 
-IdentityXPathParser.build_tokenizer()
-
-
 class XsdSelector(XsdComponent):
     """Class for defining an XPath selector for an XSD identity constraint."""
     _ADMITTED_TAGS = {XSD_SELECTOR}
     xpath_default_namespace = ''
-    pattern = re.compile(translate_pattern(
+    pattern: Union[str, Pattern[str]] = translate_pattern(
         r"(\.//)?(((child::)?((\i\c*:)?(\i\c*|\*)))|\.)(/(((child::)?"
         r"((\i\c*:)?(\i\c*|\*)))|\.))*(\|(\.//)?(((child::)?((\i\c*:)?"
         r"(\i\c*|\*)))|\.)(/(((child::)?((\i\c*:)?(\i\c*|\*)))|\.))*)*",
         back_references=False,
         lazy_quantifiers=False,
         anchors=False
-    ))
-    token = None
-    parser = None
+    )
+    token = None   # type: XPathToken
+    parser = None  # type: IdentityXPathParser
 
-    def __init__(self, elem, schema, parent):
+    def __init__(self, elem: ElementType, schema: SchemaType,
+                 parent: Optional['XsdIdentity']) -> None:
         super(XsdSelector, self).__init__(elem, schema, parent)
 
-    def _parse(self):
-        super(XsdSelector, self)._parse()
+    def _parse(self) -> None:
         try:
             self.path = self.elem.attrib['xpath']
         except KeyError:
-            self.parse_error("'xpath' attribute required:", self.elem)
+            self.parse_error(_("'xpath' attribute required"))
             self.path = '*'
         else:
-            if not self.pattern.match(self.path.replace(' ', '')):
-                self.parse_error("Wrong XPath expression for an xs:selector")
+            path = self.path.replace(' ', '')
+            try:
+                _match = self.pattern.match(path)  # type: ignore[union-attr]
+            except AttributeError:
+                # Compile regex pattern
+                self.__class__.pattern = re.compile(self.pattern)
+                _match = self.pattern.match(path)  # type: ignore[union-attr]
+
+            if not _match:
+                msg = _("invalid XPath expression for an {}")
+                self.parse_error(msg.format(self.__class__.__name__))
 
         # XSD 1.1 xpathDefaultNamespace attribute
         if self.schema.XSD_VERSION > '1.0':
@@ -101,34 +115,32 @@ class XsdSelector(XsdComponent):
         try:
             self.token = self.parser.parse(self.path)
         except ElementPathError as err:
-            self.parse_error(err)
             self.token = self.parser.parse('*')
+            self.parse_error(err)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return '%s(path=%r)' % (self.__class__.__name__, self.path)
 
     @property
-    def built(self):
+    def built(self) -> bool:
         return self.token is not None
 
     @property
-    def target_namespace(self):
-        if ':' in self.path:
-            match = QNAME_PATTERN.findall(self.path)
-            if match is not None:
-                prefix = match[0][0]
-                if prefix:
-                    return self.namespaces[prefix]
-                elif self.xpath_default_namespace:
-                    return self.xpath_default_namespace
-
+    def target_namespace(self) -> str:
+        # TODO: implement a property in elementpath for getting XPath token's namespace
+        if self.token is None:
+            pass  # xpathDefaultNamespace="##targetNamespace"
+        elif self.token.symbol == ':':
+            return self.token[1].namespace or self.xpath_default_namespace
+        elif self.token.symbol == '@' and self.token[0].symbol == ':':
+            return self.token[0][1].namespace or self.xpath_default_namespace
         return self.schema.target_namespace
 
 
 class XsdFieldSelector(XsdSelector):
     """Class for defining an XPath field selector for an XSD identity constraint."""
     _ADMITTED_TAGS = {XSD_FIELD}
-    pattern = re.compile(translate_pattern(
+    pattern = translate_pattern(
         r"(\.//)?((((child::)?((\i\c*:)?(\i\c*|\*)))|\.)/)*((((child::)?"
         r"((\i\c*:)?(\i\c*|\*)))|\.)|((attribute::|@)((\i\c*:)?(\i\c*|\*))))"
         r"(\|(\.//)?((((child::)?((\i\c*:)?(\i\c*|\*)))|\.)/)*"
@@ -137,7 +149,7 @@ class XsdFieldSelector(XsdSelector):
         back_references=False,
         lazy_quantifiers=False,
         anchors=False
-    ))
+    )
 
 
 class XsdIdentity(XsdComponent):
@@ -147,85 +159,94 @@ class XsdIdentity(XsdComponent):
     :ivar selector: the XPath selector of the identity constraint.
     :ivar fields: a list containing the XPath field selectors of the identity constraint.
     """
-    selector = None
-    elements = None  # XSD elements bound by selector (for speed-up and lazy mode)
-    fields = ()
+    name: str
+    local_name: str
+    prefixed_name: str
+    parent: 'XsdElement'
+    ref: Optional['XsdIdentity']
 
-    def __init__(self, elem, schema, parent):
+    selector = None  # type: XsdSelector
+    fields = ()      # type: Union[Tuple[()], List[XsdFieldSelector]]
+
+    # XSD elements bound by selector (for speed-up and for lazy mode)
+    elements: Union[Tuple[()], Dict['XsdElement', Optional[IdentityCounterType]]] = ()
+
+    def __init__(self, elem: ElementType, schema: SchemaType,
+                 parent: Optional['XsdElement']) -> None:
         super(XsdIdentity, self).__init__(elem, schema, parent)
 
-    def _parse(self):
-        super(XsdIdentity, self)._parse()
-        elem = self.elem
+    def _parse(self) -> None:
         try:
-            self.name = get_qname(self.target_namespace, elem.attrib['name'])
+            self.name = get_qname(self.target_namespace, self.elem.attrib['name'])
         except KeyError:
-            self.parse_error("missing required attribute 'name'", elem)
-            self.name = None
+            self.parse_error(_("missing required attribute 'name'"))
+            self.name = ''
 
-        for index, child in enumerate(elem):
+        for child in self.elem:
             if child.tag == XSD_SELECTOR:
                 self.selector = XsdSelector(child, self.schema, self)
                 break
-            elif child.tag != XSD_ANNOTATION:
-                self.parse_error("'selector' declaration expected.", elem)
-                break
         else:
-            self.parse_error("missing 'selector' declaration.", elem)
-            index = -1
+            self.parse_error(_("missing 'selector' declaration"))
 
         self.fields = []
-        for child in filter(is_not_xsd_annotation, elem[index + 1:]):
+        for child in self.elem:
             if child.tag == XSD_FIELD:
                 self.fields.append(XsdFieldSelector(child, self.schema, self))
-            else:
-                self.parse_error("%r is not allowed here" % child, elem)
 
-    def _parse_identity_reference(self):
-        super(XsdIdentity, self)._parse()
-        self.name = get_qname(self.target_namespace, self.elem.attrib['ref'])
-        if 'name' in self.elem.attrib:
-            self.parse_error("attributes 'name' and 'ref' are mutually exclusive")
-        elif self._parse_child_component(self.elem) is not None:
-            self.parse_error("a reference cannot has child definitions")
-
-    def build(self):
-        if self.ref is True:
+    def build(self) -> None:
+        if self.ref is True:  # type: ignore[comparison-overlap]
             try:
                 ref = self.maps.identities[self.name]
             except KeyError:
-                self.parse_error("Unknown identity constraint {!r}".format(self.name))
+                msg = _("unknown identity constraint {!r}")
+                self.parse_error(msg.format(self.name))
                 return
             else:
                 if not isinstance(ref, self.__class__):
-                    self.parse_error("attribute 'ref' points to a different kind constraint")
+                    msg = _("attribute 'ref' points to a different kind constraint")
+                    self.parse_error(msg)
                 self.selector = ref.selector
                 self.fields = ref.fields
                 self.ref = ref
 
-        context = IdentityXPathContext(self.schema, item=self.parent)
+        context = IdentityXPathContext(self.schema, item=self.parent)  # type: ignore
 
-        self.elements = {
-            e: None for e in self.selector.token.select_results(context) if e.name
-        }
+        self.elements = {}
+        try:
+            for e in self.selector.token.select_results(context):
+                if not isinstance(e, XsdComponent) or isinstance(e, XsdAttribute):
+                    msg = _("selector xpath expression can only select elements")
+                    self.parse_error(msg)
+                elif e.name is not None:
+                    if TYPE_CHECKING:
+                        assert isinstance(e, XsdElement)  # for mypy checks with Python 3.7
+                    self.elements[e] = None
+        except AttributeError:
+            pass
+        else:
+            if not self.elements:
+                # Try to detect target XSD elements extracting QNames
+                # of the leaf elements from the XPath expression and
+                # use them to match global elements.
 
-        if any(isinstance(e, XsdAttribute) for e in self.elements):
-            self.parse_error("selector xpath cannot select attributes")
-        elif not self.elements:
-            # Try to detect target XSD elements extracting QNames
-            # of the leaf elements from the XPath expression and
-            # use them to match global elements.
-
-            for qname in self.selector.token.iter_leaf_elements():
-                xsd_element = self.maps.elements.get(get_extended_qname(qname, self.namespaces))
-                if xsd_element is not None and xsd_element not in self.elements:
-                    self.elements[xsd_element] = None
+                qname: Any
+                for qname in self.selector.token.iter_leaf_elements():
+                    xsd_element = self.maps.elements.get(
+                        get_extended_qname(qname, self.namespaces)
+                    )
+                    if xsd_element is not None and \
+                            not isinstance(xsd_element, tuple) and \
+                            xsd_element not in self.elements:
+                        self.elements[xsd_element] = None
 
     @property
-    def built(self):
-        return self.elements is not None
+    def built(self) -> bool:
+        return not isinstance(self.elements, tuple)
 
-    def get_fields(self, elem, namespaces=None, decoders=None):
+    def get_fields(self, elem: Union[ElementType, 'XsdElement'],
+                   namespaces: Optional[NamespacesType] = None,
+                   decoders: Optional[Tuple[XsdAttribute, ...]] = None) -> IdentityCounterType:
         """
         Get fields for a schema or instance context element.
 
@@ -234,14 +255,18 @@ class XsdIdentity(XsdComponent):
         :param decoders: context schema fields decoders.
         :return: a tuple with field values. An empty field is replaced by `None`.
         """
-        fields = []
-        if isinstance(elem, XsdComponent):
-            context_class = IdentityXPathContext
-        else:
-            context_class = XPathContext
+        fields: List[IdentityFieldItemType] = []
 
+        if not isinstance(elem, XsdComponent):
+            context_class = XPathContext
+        else:
+            context_class = IdentityXPathContext
+
+        result: Any
+        value: Union[AtomicValueType, None]
         for k, field in enumerate(self.fields):
-            result = field.token.get_results(context_class(elem))
+            result = field.token.get_results(context_class(elem))  # type: ignore
+
             if not result:
                 if decoders is not None and decoders[k] is not None:
                     value = decoders[k].value_constraint
@@ -268,18 +293,23 @@ class XsdIdentity(XsdComponent):
                 elif field.target_namespace not in self.maps.namespaces:
                     fields.append(None)
                 else:
-                    raise XMLSchemaValueError("%r key field must have a value!" % field)
+                    msg = _("missing key field {0!r} for {1!r}")
+                    raise XMLSchemaValueError(msg.format(field.path, self))
 
             elif len(result) == 1:
                 if decoders is None or decoders[k] is None:
                     fields.append(result[0])
                 else:
                     if decoders[k].type.content_type_label not in ('simple', 'mixed'):
-                        raise XMLSchemaTypeError("%r field doesn't have a simple type!" % field)
+                        msg = _("%r field doesn't have a simple type!")
+                        raise XMLSchemaTypeError(msg % field)
 
                     value = decoders[k].data_value(result[0])
                     if decoders[k].type.root_type.name == XSD_QNAME:
-                        value = get_extended_qname(value, namespaces)
+                        if isinstance(value, str):
+                            value = get_extended_qname(value, namespaces)
+                        elif isinstance(value, datatypes.QName):
+                            value = value.expanded_name
 
                     if isinstance(value, list):
                         fields.append(tuple(value))
@@ -292,11 +322,12 @@ class XsdIdentity(XsdComponent):
                     else:
                         fields.append((value, float))
             else:
-                raise XMLSchemaValueError("%r field selects multiple values!" % field)
+                msg = _("%r field selects multiple values!")
+                raise XMLSchemaValueError(msg % field)
 
         return tuple(fields)
 
-    def get_counter(self, enabled=True):
+    def get_counter(self, enabled: bool = True) -> 'IdentityCounter':
         return IdentityCounter(self, enabled)
 
 
@@ -316,20 +347,20 @@ class XsdKeyref(XsdIdentity):
     or in a descendant element.
     """
     _ADMITTED_TAGS = {XSD_KEYREF}
-    refer = None
+    refer: Optional[Union[str, XsdKey]] = None
     refer_path = '.'
 
-    def _parse(self):
+    def _parse(self) -> None:
         super(XsdKeyref, self)._parse()
         try:
             self.refer = self.schema.resolve_qname(self.elem.attrib['refer'])
         except (KeyError, ValueError, RuntimeError) as err:
             if 'refer' not in self.elem.attrib:
-                self.parse_error("missing required attribute 'refer'")
+                self.parse_error(_("missing required attribute 'refer'"))
             else:
                 self.parse_error(err)
 
-    def build(self):
+    def build(self) -> None:
         super(XsdKeyref, self).build()
 
         if isinstance(self.refer, (XsdKey, XsdUnique)):
@@ -342,18 +373,21 @@ class XsdKeyref(XsdIdentity):
         elif isinstance(self.refer, str):
             refer = self.parent.identities.get(self.refer)
             if refer is not None and refer.ref is None:
-                self.refer = refer
+                self.refer = refer  # type: ignore[assignment]
             else:
                 try:
-                    self.refer = self.maps.identities[self.refer]
+                    self.refer = self.maps.identities[self.refer]  # type: ignore[assignment]
                 except KeyError:
-                    self.parse_error("key/unique identity constraint %r is missing" % self.refer)
+                    msg = _("key/unique identity constraint %r is missing")
+                    self.parse_error(msg % self.refer)
                     return
 
         if not isinstance(self.refer, (XsdKey, XsdUnique)):
-            self.parse_error("reference to a non key/unique identity constraint %r" % self.refer)
+            msg = _("reference to a non key/unique identity constraint %r")
+            self.parse_error(msg % self.refer)
         elif len(self.refer.fields) != len(self.fields):
-            self.parse_error("field cardinality mismatch between %r and %r" % (self, self.refer))
+            msg = _("field cardinality mismatch between {0!r} and {1!r}")
+            self.parse_error(msg.format(self, self.refer))
         elif self.parent is not self.refer.parent:
             refer_path = self.refer.parent.get_path(ancestor=self.parent)
             if refer_path is None:
@@ -365,101 +399,82 @@ class XsdKeyref(XsdIdentity):
                 # needs to propagate upwards, with conflict resolution."
                 refer_path = self.parent.get_path(ancestor=self.refer.parent, reverse=True)
                 if refer_path is None:
-                    refer_path = self.parent.get_path(reverse=True) + '/' + \
-                        self.refer.parent.get_path()
+                    path1 = self.parent.get_path(reverse=True)
+                    path2 = self.refer.parent.get_path()
+                    assert path1 is not None
+                    assert path2 is not None
+                    refer_path = f'{path1}/{path2}'
 
             self.refer_path = refer_path
 
     @property
-    def built(self):
-        return self.elements is not None and isinstance(self.refer, XsdIdentity)
+    def built(self) -> bool:
+        return not isinstance(self.elements, tuple) and isinstance(self.refer, XsdIdentity)
 
-    def __call__(self, identities, elem, namespaces=None):
-        if self.refer is None:
-            return
-
-        values = identities[self].counter
-        refer_values = identities[self.refer].counter
-
-        for v in values:
-            if v not in refer_values:
-                reason = "Value {!r} not found for key {} ({} times)" \
-                    .format(v, self.refer.prefixed_name, values[v])
-                yield XMLSchemaValidationError(validator=self, obj=elem, reason=reason)
-
-    def get_counter(self, enabled=True):
+    def get_counter(self, enabled: bool = True) -> 'KeyrefCounter':
         return KeyrefCounter(self, enabled)
 
 
 class Xsd11Unique(XsdUnique):
-
-    def _parse(self):
+    def _parse(self) -> None:
         if self._parse_reference():
-            super(XsdIdentity, self)._parse()
-            self.ref = True
+            self.ref = True  # type: ignore[assignment]
         else:
             super(Xsd11Unique, self)._parse()
 
 
 class Xsd11Key(XsdKey):
-
-    def _parse(self):
+    def _parse(self) -> None:
         if self._parse_reference():
-            super(XsdIdentity, self)._parse()
-            self.ref = True
+            self.ref = True  # type: ignore[assignment]
         else:
             super(Xsd11Key, self)._parse()
 
 
 class Xsd11Keyref(XsdKeyref):
-
-    def _parse(self):
+    def _parse(self) -> None:
         if self._parse_reference():
-            super(XsdIdentity, self)._parse()
-            self.ref = True
+            self.ref = True  # type: ignore[assignment]
         else:
             super(Xsd11Keyref, self)._parse()
 
 
-class IdentityCounter(object):
+class IdentityCounter:
 
-    def __init__(self, identity, enabled=True):
-        self.counter = Counter()
+    def __init__(self, identity: XsdIdentity, enabled: bool = True) -> None:
+        self.counter: Counter[IdentityCounterType] = Counter[IdentityCounterType]()
         self.identity = identity
         self.enabled = enabled
 
-    def __repr__(self):
-        return "%s(counter=%r)" % (self.__class__.__name__, self.counter)
+    def __repr__(self) -> str:
+        return "%s%r" % (self.__class__.__name__[:-7], self.counter)
 
-    def clear(self):
+    def clear(self) -> None:
         self.counter.clear()
         self.enabled = True
 
-    def increase(self, fields):
+    def increase(self, fields: IdentityCounterType) -> None:
         self.counter[fields] += 1
         if self.counter[fields] == 2:
-            msg = "duplicated value {!r} for {!r}"
+            msg = _("duplicated value {0!r} for {1!r}")
             raise XMLSchemaValueError(msg.format(fields, self.identity))
 
 
 class KeyrefCounter(IdentityCounter):
+    identity: XsdKeyref
 
-    def increase(self, fields):
+    def increase(self, fields: IdentityCounterType) -> None:
         self.counter[fields] += 1
 
-    def iter_errors(self, identities):
-        try:
-            refer_values = identities[self.identity.refer].counter
-        except KeyError:
-            if self.identity.refer is not None:
-                raise
-        else:
-            for v in filter(lambda x: x not in refer_values, self.counter):
-                if len(v) == 1 and v[0] in refer_values:
-                    continue
-                elif self.counter[v] > 1:
-                    msg = "Value {} not found for {!r} ({} times)"
-                    yield XMLSchemaValueError(msg.format(v, self.identity.refer, self.counter[v]))
-                else:
-                    msg = "Value {} not found for {!r}"
-                    yield XMLSchemaValueError(msg.format(v, self.identity.refer))
+    def iter_errors(self, identities: IdentityMapType) -> Iterator[XMLSchemaValueError]:
+        refer_values = identities[self.identity.refer].counter
+
+        for v in filter(lambda x: x not in refer_values, self.counter):
+            if len(v) == 1 and v[0] in refer_values:
+                continue
+            elif self.counter[v] > 1:
+                msg = "value {} not found for {!r} ({} times)"
+                yield XMLSchemaValueError(msg.format(v, self.identity.refer, self.counter[v]))
+            else:
+                msg = "value {} not found for {!r}"
+                yield XMLSchemaValueError(msg.format(v, self.identity.refer))
